require 'spec_helper'

enum :array_colors, [:yellow, "green", 'blue']
enum :hash_colors, {:yellow => 2, "green" => 5, 'blue' => 8, :mauve => 20}
enum :module_method_names, [:name, "display", 'clone']

describe Enumz do

  describe 'creation' do

    describe 'via array' do
      names = [:yellow, "green", 'blue']
      values = [1,2,3]

      it_should_behave_like 'a properly created enum', ArrayColors, 3, names, values

      it 'should generate values' do
        ArrayColors.values.should_not be_nil
        ArrayColors.values.count.should eq(3)
        ArrayColors.values.reduce(1) do |counter, value|
          counter.should eq(value)
          counter += 1
        end
      end

    end # creation

    describe 'via hash' do
      names = [:yellow, "green", 'blue', :mauve]
      values = [2,5,8,20]

      it_should_behave_like 'a properly created enum', HashColors, 4, names, values
    end

  end # via hash

  describe "features" do

    it 'should remove module methods to avoid potential conflicts when creating methods for enum names' do
      ModuleMethodNames.name.should eq(1)
      ModuleMethodNames.display.should eq(2)
      ModuleMethodNames.clone.should eq(3)
    end

  end

  describe 'exceptions' do
    module ExceptionsEnum
      extend Enumz
    end

    it 'should raise an exception if enum name is not a symbol' do
      expect { ExceptionsEnum.enum "hi", [] }.to raise_error ArgumentError
    end

    it 'should raise an exception if enum values is neither a hash nor an array' do
      expect { ExceptionsEnum.enum :test_enum, :invalid_value }.to raise_error ArgumentError
    end

    it 'should raise an exception if attempting to access a non-existent enum name' do
      expect { ArrayColors.invisible }.to raise_error NoMethodError
    end

    it 'should raise an exception if an invalid enum name is provided' do
      expect { ExceptionsEnum.enum :ages_enum, {:'20_years' => 20} }.to raise_error NameError
    end

    it 'should raise an exception if a duplicate enum value is supplied' do
      expect { ExceptionsEnum.enum :test_enum, {:yellow => 1, :green => 2, :blue => 1}}.to raise_error ArgumentError
    end

    it 'should raise an exception when creating an enum with a name that is a ruby reserved method name' do
      BaseEnumz::RESERVED_METHOD_NAMES.each do |name|
        expect { ExceptionsEnum.enum :test_enum, [name] }.to raise_error ArgumentError
      end
    end

    it 'should raise a read-only exception when trying to update' do
      expect { ArrayColors.data_elements[:alyssum] = 100 }.to raise_error RuntimeError
    end

  end # validations

end # Enumz
