shared_examples "a properly created enum" do |enum, num_data_elements, names, values|

  it 'should create a module whose name matches that of the enum' do
    enum.should_not be_nil
    enum.class.should eq(Module)
  end

  it 'should be valid' do
    enum.data_elements.class.should eq(Hash)
    enum.data_elements.count.should eq(num_data_elements)

    enum.names.count.should eq(num_data_elements)
    enum.values.count.should eq(num_data_elements)

    # compare actual names w/ expected names
    enum.names.reduce(0) do |counter, key|
      names[counter].to_sym.should eq(key)
      counter += 1
    end

    # compare actual values w/ expected values
    enum.values.reduce(0) do |counter, value|
      values[counter].should eq(value)
      counter += 1
    end
  end

  it 'should symbolize all names' do
    enum.names.each do |key|
      key.class.should eq(Symbol)
    end
  end

  it 'should access values via methods whose names match those of the enum names' do
    enum.names.reduce(0) do |counter, key|
      enum.send("#{key}").should eq(values[counter])
      counter += 1
    end
  end

  it 'should access values via constants whose names match those of the enum names' do
    enum.names.reduce(0) do |counter, key|
      enum.const_get(key.to_s.capitalize).should eq(values[counter])
      counter += 1
    end
  end

  it 'should get enum name by value' do
    values.reduce(0) do |counter, value|
      enum[value].should eq(names[counter].to_sym)
      counter += 1
    end
  end

end
